"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
function _export(target, all) {
    for(var name in all)Object.defineProperty(target, name, {
        enumerable: true,
        get: all[name]
    });
}
_export(exports, {
    UploadPlugin: function() {
        return UploadPlugin;
    },
    default: function() {
        return _default;
    }
});
const _define_property = require("@swc/helpers/_/_define_property");
const _interop_require_default = require("@swc/helpers/_/_interop_require_default");
const _state = require("@codemirror/state");
const _view = require("@codemirror/view");
const _consts = require("./consts");
const _state1 = require("./state");
const _UploadWidget = /*#__PURE__*/ _interop_require_default._(require("./UploadWidget"));
const _utils = require("./utils");
const DEFAULT_OPTIONS = {
    multiple: true,
    accept: 'image/*',
    generateId: _utils.generateId
};
let UploadPlugin = class UploadPlugin {
    handleUploadFile(id) {
        const item = _consts.FILE_CACHE.get(id);
        if (!item) {
            return;
        }
        const { file } = item;
        this.options.action({
            id,
            file,
            callback: {
                progress: (percent)=>{
                    item.progress = percent;
                    this.view.dispatch({
                        effects: _state1.uploadActionEffect.of({
                            id,
                            action: 'progress',
                            percent
                        })
                    });
                },
                success: (url)=>{
                    item.status = 'success';
                    item.url = url;
                    this.view.dispatch({
                        effects: _state1.uploadActionEffect.of({
                            id,
                            action: 'success',
                            url
                        })
                    });
                },
                fail: (error)=>{
                    item.status = 'error';
                    this.view.dispatch({
                        effects: _state1.uploadActionEffect.of({
                            id,
                            action: 'fail',
                            message: error.message
                        })
                    });
                }
            }
        });
    }
    async uploadFiles(files, position) {
        const lastLine = this.view.state.doc.line(this.view.state.doc.lines);
        const pos = position || {
            anchor: lastLine.to,
            head: lastLine.to
        };
        const [from, to] = [
            pos.anchor,
            pos.head
        ].sort();
        const text = pos.head === pos.anchor ? '' : this.view.state.doc.sliceString(from, to);
        const [lineBefore, lineAfter] = (()=>{
            const fromLine = this.view.lineBlockAt(from);
            const toLine = this.view.lineBlockAt(to);
            if (fromLine.from === toLine.from) {
                const charBefore = this.view.state.doc.sliceString(from - 1, from);
                const charEnd = this.view.state.doc.sliceString(to, to + 1);
                if (charBefore === '[' && charEnd === ']') {
                    return [
                        false,
                        false
                    ];
                }
            }
            if (text) {
                return [
                    false,
                    false
                ];
            }
            const startOfLine = fromLine.from === from;
            const endOfLine = toLine.to === to;
            return [
                !startOfLine,
                !endOfLine
            ];
        })();
        const imageTags = [];
        const list = [];
        for (const file of files){
            const id = await this.options.generateId(file);
            if (_consts.FILE_CACHE.has(id)) {
                const item = _consts.FILE_CACHE.get(id);
                if (item.status === 'success') {
                    imageTags.push(`![${text || file.name}](${item.url})`);
                    continue;
                }
                if (item.status === 'uploading') {
                    imageTags.push(`![${text || file.name}](${_consts.IMAGE_UPLOADING_FLAG}:${id})`);
                    continue;
                }
                if (item.status === 'error') {
                    _consts.FILE_CACHE.delete(id);
                }
            }
            _consts.FILE_CACHE.set(id, {
                file,
                progress: 0,
                status: 'uploading'
            });
            list.push(id);
            imageTags.push(`![${text || file.name}](${_consts.IMAGE_UPLOADING_FLAG}:${id})`);
        }
        const texts = [
            ...imageTags
        ];
        // make new line
        lineBefore && texts.unshift('');
        lineAfter && texts.push('');
        const imageTag = _state.Text.of(texts);
        this.view.dispatch({
            changes: [
                {
                    from,
                    to,
                    insert: imageTag
                }
            ],
            selection: _state.EditorSelection.cursor(from + imageTag.length)
        });
        await Promise.all(list.map((id)=>this.handleUploadFile(id)));
    }
    createSelector() {
        if (this.selectTrigger) {
            this.selectTrigger.parentNode?.removeChild(this.selectTrigger);
        }
        const selector = window.document.createElement('input');
        this.selectTrigger = selector;
        selector.type = 'file';
        selector.accept = this.options.accept || 'image/*';
        selector.multiple = this.options.multiple || false;
        selector.style.display = 'none';
        this.view.dom.appendChild(selector);
    }
    async selectFile() {
        if (!this.selectTrigger) {
            this.createSelector();
        }
        return new Promise((resolve)=>{
            this.selectTrigger.addEventListener('change', (event)=>{
                const files = event.target.files;
                resolve(Array.from(files || []));
                this.selectTrigger.value = '';
            }, {
                once: true
            });
            this.selectTrigger?.click();
        });
    }
    // update
    uploadStateChanged(states) {
        if (!states.size) {
            return;
        }
        const ids = Array.from(states.keys());
        // todo: support multi match in single line
        const MATCH_REGEXP = new RegExp(`(!\\[.*]\\(${_consts.IMAGE_UPLOADING_FLAG}:(?<id>(${ids.join('|')}))\\))`, 'i');
        const { doc } = this.view.state;
        const lineCount = doc.lines;
        const changes = [];
        for(let i = 1; i <= lineCount; i++){
            const line = doc.line(i);
            const { text } = line;
            if (!text) {
                continue;
            }
            const matched = text.match(MATCH_REGEXP);
            if (!matched) {
                continue;
            }
            const id = matched.groups.id;
            const newState = states.get(id);
            const file = _consts.FILE_CACHE.get(id);
            switch(newState.action){
                case 'success':
                    {
                        const newText = text.replace(`${_consts.IMAGE_UPLOADING_FLAG}:${id}`, newState.url);
                        changes.push({
                            from: line.from + (matched.index || 0),
                            to: line.from + matched.index + matched[0].length,
                            insert: newText
                        });
                        file.status = 'success';
                        file.url = newState.url;
                        _consts.FILE_CACHE.set(id, file);
                        break;
                    }
                case 'fail':
                    {
                        const file = _consts.FILE_CACHE.get(id);
                        file.status = 'error';
                        file.error = newState.message;
                        _consts.FILE_CACHE.set(id, file);
                        break;
                    }
            }
            if ([
                'progress',
                'fail'
            ].includes(newState.action)) {
                this.decorations = this.decorations.update({
                    filterFrom: line.from,
                    filterTo: line.to,
                    filter: ()=>{
                        return false;
                    },
                    add: [
                        {
                            from: line.from + matched.index + matched[0].length,
                            to: line.from + matched.index + matched[0].length,
                            value: _view.Decoration.widget({
                                widget: new _UploadWidget.default(id)
                            })
                        }
                    ]
                });
            } else {
                this.decorations = this.decorations.update({
                    filterFrom: line.from,
                    filterTo: line.to,
                    filter: ()=>{
                        return false;
                    }
                });
            }
        }
        if (changes.length) {
            this.view.dispatch({
                changes: changes.sort((a, b)=>a.from - b.from)
            });
        }
    }
    renderWidgets() {
        const decorations = new _state.RangeSetBuilder();
        const { state } = this.view;
        const { doc } = state;
        const lineCount = doc.lines;
        for(let i = 1; i <= lineCount; i++){
            const line = doc.line(i);
            const { text } = line;
            if (!text) {
                continue;
            }
            const matched = text.match(_consts.IMAGE_UPLOADING_CACHE_REGEXP);
            if (!matched) {
                continue;
            }
            const id = matched.groups.id;
            const item = _consts.FILE_CACHE.get(id);
            if (!item) {
                continue;
            }
            decorations.add(line.from + matched.index + matched[0].length, line.from + matched.index + matched[0].length, _view.Decoration.widget({
                widget: new _UploadWidget.default(id)
            }));
        }
        this.decorations = decorations.finish();
    }
    update(update) {
        const changedFiles = new Map;
        update.transactions.forEach((transaction)=>{
            transaction.effects.forEach(async (effect)=>{
                if (effect.is(_state1.selectFileTriggerEffect)) {
                    const files = await this.selectFile();
                    files.length && await this.uploadFiles(files, effect.value || undefined);
                    return;
                }
                if (effect.is(_state1.uploadActionEffect)) {
                    const { value } = effect;
                    const { id } = value;
                    changedFiles.set(id, value);
                }
            });
        });
        if (changedFiles.size) {
            setTimeout(()=>{
                this.uploadStateChanged(changedFiles);
            });
        }
        if (update.docChanged) {
            this.renderWidgets();
        }
    }
    constructor(view, options){
        _define_property._(this, "view", void 0);
        _define_property._(this, "decorations", void 0);
        _define_property._(this, "options", void 0);
        _define_property._(this, "selectTrigger", void 0);
        this.view = view;
        this.decorations = _view.Decoration.none;
        this.selectTrigger = undefined;
        this.options = {
            ...DEFAULT_OPTIONS,
            ...options
        };
        this.renderWidgets();
    }
};
const uploadPlugin = (options)=>{
    return _view.ViewPlugin.define((view)=>new UploadPlugin(view, options), {
        decorations: (v)=>v.decorations,
        provide: (v)=>{
            return _view.EditorView.atomicRanges.of((view)=>{
                return view.plugin(v)?.decorations || _view.Decoration.none;
            });
        }
    });
};
const _default = uploadPlugin;
